"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgRelayPreset = exports.PgRelayPlugin = void 0;
require("graphile-config");
require("graphile-build-pg");
const version_js_1 = require("../version.js");
const RELAY_HIDDEN_COLUMN_BEHAVIORS = [
    "-select",
    "-update",
    "-base",
    "-filterBy",
    "-orderBy",
];
/** @experimental */
exports.PgRelayPlugin = {
    name: "RelayPlugin",
    description: "[EXPERIMENTAL] Adds behaviors, inflectors, and other accomodations to better hone your schema for Relay usage",
    version: version_js_1.version,
    after: ["PostGraphileV4CompatibilityPlugin"],
    inflection: {
        replace: {
            nodeIdFieldName() {
                return "id";
            },
            attribute(previous, options, details) {
                const name = previous.call(this, details);
                if (name === "id")
                    return "rowId";
                return name;
            },
        },
        ignoreReplaceIfNotExists: ["attribute"],
    },
    schema: {
        globalBehavior: `\
+node \
+connection -list \
-query:resource:single \
+nodeId:filterBy \
+nodeId:resource:update -constraint:resource:update \
+nodeId:resource:delete -constraint:resource:delete \
+nodeId:insert -relation:attribute:insert \
+nodeId:update -relation:attribute:update \
+nodeId:base -relation:attribute:base \
`,
        entityBehavior: {
            pgCodecAttribute(behavior, [codec, attributeName], build) {
                const newBehavior = [behavior];
                const resource = Object.values(build.input.pgRegistry.pgResources).find((r) => {
                    if (r.codec !== codec)
                        return false;
                    if (r.parameters)
                        return false;
                    if (r.isVirtual)
                        return false;
                    if (r.isUnique)
                        return false;
                    if (r.uniques.length === 0)
                        return false;
                    return true;
                });
                const pk = resource?.uniques.find((u) => u.isPrimary);
                // If the column is a primary key, don't include it (since it will be in the NodeID instead)
                if (pk?.attributes.includes(attributeName)) {
                    // Do not include this column in the schema (other than for create)
                    newBehavior.push(...RELAY_HIDDEN_COLUMN_BEHAVIORS);
                }
                else {
                    // If the column is available via a singular relation, don't include the column itself
                    const relationsMap = build.input.pgRegistry.pgRelations[codec.name];
                    const relations = relationsMap
                        ? Object.values(build.input.pgRegistry.pgRelations[codec.name])
                        : [];
                    const singularRelationsUsingThisColumn = relations.filter((r) => {
                        // TODO: check relation is visible
                        if (!r.isUnique)
                            return false;
                        if (r.isReferencee)
                            return false;
                        if (!r.localAttributes.includes(attributeName))
                            return false;
                        return true;
                    });
                    if (singularRelationsUsingThisColumn.length > 0) {
                        // Do not include this column in the schema (other than for create)
                        newBehavior.push(...RELAY_HIDDEN_COLUMN_BEHAVIORS);
                    }
                }
                return newBehavior;
            },
        },
    },
};
/** @experimental */
exports.PgRelayPreset = {
    plugins: [exports.PgRelayPlugin],
    schema: {
        pgMutationPayloadRelations: false,
        pgFunctionsPreferNodeId: true,
    },
};
//# sourceMappingURL=relay.js.map